<?php

namespace Drupal\like\Plugin\views\argument_default;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\views\Plugin\views\argument_default\ArgumentDefaultPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default argument plugin to use the raw value from the URL.
 *
 * @ingroup views_argument_default_plugins
 *
 * @ViewsArgumentDefault(
 *   id = "like_user_cookie",
 *   title = @Translation("Like: owner by user or cookie")
 * )
 */
class LikeUserOrCookie extends ArgumentDefaultPluginBase implements CacheableDependencyInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a LikeUserOrCookie object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, AccountProxyInterface $current_user, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getArgument() {
    $nids = [];
    $entity_type_id = $this->argument->getPluginDefinition()['provider'] ?: 'node';
    $cookie_value = $_COOKIE['Drupal_visitor_like'] ?? NULL;
    // Get the node ids from the cookie no matter if the user is logged in or
    // not.
    if (!empty($cookie_value)) {
      $cookie_value = Json::decode($cookie_value);
      $nids = array_merge($nids, $cookie_value[$entity_type_id]);
    }

    if ($this->currentUser->isAuthenticated()) {
      $storage = $this->entityTypeManager->getStorage('like');
      $likes = $storage
        ->getQuery()
        ->accessCheck()
        ->condition('entity_type', $entity_type_id)
        ->condition('uid', $this->currentUser->id())
        ->execute();
      $likes = $storage->loadMultiple($likes);
      foreach ($likes as $like) {
        $nids[] = $like->get('entity_id')->target_id;
      }
    }
    return empty($nids) ? '' : implode('+', array_unique($nids));
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return Cache::PERMANENT;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return ['user', 'cookies: Drupal_visitor_like'];
  }

}
